@@ -38,7 +38,7 @@ sub main {
 
 	my $params = FAQ::OMatic::getParams($cgi);
 
-	$rt .= FAQ::OMatic::pageHeader();
+	$rt .= FAQ::OMatic::pageHeader($params);
 	
 	$item = new FAQ::OMatic::Item($params->{'file'});
 	if ($item->isBroken()) {
@@ -46,6 +46,10 @@ sub main {
 			.") doesn't exist.");
 	}
 
+	# check sequence number to prevent later confusion -- the user's
+	# insert or change request came from an out-of-date page.
+	$item->checkSequence($params);
+
 	my $insertpart = $params->{'_insertpart'};
 
 	my $partnum = $params->{'partnum'};
@@ -101,12 +105,21 @@ sub main {
 		my $rd = FAQ::OMatic::Auth::ensurePerm($item, 'PermEditPart',
 			FAQ::OMatic::commandName(), $cgi, 0);
 		if ($rd) { print $rd; exit 0; }
+
+		if ($part->{'Type'} eq 'html') {
+			# discourage unauthorized users from editing HTML parts which
+			# they won't later be able to submit.
+			$rd = FAQ::OMatic::Auth::ensurePerm($item, 'PermUseHTML',
+				FAQ::OMatic::commandName(), $cgi, 0, 'useHTML');
+			if ($rd) { print $rd; exit 0; }
+		}
 	}
 	
 	if ($params->{'_insertpart'}) {
-		if ($params->{'_insert'} eq 'answer') {
+		my $insertHint = $params->{'_insert'} || '';
+		if ($insertHint eq 'answer') {
 			$rt .= "Enter the answer to <b>".$item->getTitle()."</b>\n";
-		} elsif ($params->{'_insert'} eq 'category') {
+		} elsif ($insertHint eq 'category') {
 			$rt .= "Enter a description for <b>".$item->getTitle()."</b>\n";
 		} elsif ($params->{'_duplicate'}) {
 			$rt .= "Edit duplicated text for <b>".$item->getTitle()."</b>\n";
@@ -121,7 +134,11 @@ sub main {
 	}
 	$rt .= $part->displayPartEditor($item, $partnum, $params);
 
-	$rt .= FAQ::OMatic::pageFooter();
+	$rt .= FAQ::OMatic::Help::helpFor($params, 'editPart', "<br>\n");
+	$rt .= FAQ::OMatic::Help::helpFor($params, 'makingLinks', "<br>\n");
+	$rt .= FAQ::OMatic::Help::helpFor($params, 'seeAlso', "<br>\n");
+
+	$rt .= FAQ::OMatic::pageFooter($params, ['help','faq']);
 
 	print $rt;
 }
